use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007002
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib  ) );
__DATA__
drolsky
DROLSKY
DROLSKY's
PayPal
Rolsky
Rolsky's
java
lowercasing
markup
PDF
pl
plantuml
PlantUML
renderer
Renderers
renderers
uppercasing
Dave
autarch
Mark
Fowler
mark
bin
graph
lib
Meta
Grapher
Moose
CommandLine
Constants
Renderer
Graphviz
Plantuml
Class
Link
Role
HasOutput
